import crypto from 'crypto'

const AUTH_MODE = [
  'aes-128-gcm',
  'aes-192-gcm',
  'aes-256-gcm',
  'aes-128-ccm',
  'aes-192-ccm',
  'aes-256-ccm',
  'aes-128-ocb',
  'aes-192-ocb',
  'aes-256-ocb'
]
const VERSION = +process.versions.node.split('.').slice(0, 2).join('.')

// <Buffer d7 2c 87 d0 f0 77 c7 76 6f 29 85 df ab 30 e8 95>
const EMPTY_KEY = crypto.scryptSync
  ? crypto.scryptSync('', '', 16)
  : Buffer.from(
      '0xd7 0x2c 0x87 0xd0 0xf0 0x77 0xc7 0x76 0x6f 0x29 0x85 0xdf 0xab 0x30 0xe8 0x95'.split(
        ' '
      )
    )
//
if (!crypto.randomUUID) {
  crypto.randomUUID = function () {
    let str = crypto.randomBytes(16).toString('hex')
    return (
      str.slice(0, 8) +
      '-' +
      str.slice(8, 12) +
      '-' +
      str.slice(12, 16) +
      '-' +
      str.slice(16, 20) +
      '-' +
      str.slice(-12)
    )
  }
}
if (!crypto.randomInt) {
  crypto.randomInt = function (max) {
    if (max === void 0) {
      throw new TypeError(
        '[ERR_INVALID_ARG_TYPE]: The "max" argument must be a safe integer. Received undefined'
      )
    }
    return ~~(Math.random() * max)
  }
}

export const origin = crypto

export const hash = function (mode, data, outEncode) {
  let sum = crypto.createHash(mode)
  let isBuffer = Buffer.isBuffer(data)

  sum.update(data, isBuffer ? 'binary' : 'utf8')
  return sum.digest(outEncode || 'hex')
}

export const hmac = function (mode, data, key, outEncode) {
  key = key || ''
  let sum = crypto.createHmac(mode, key)
  let isBuffer = Buffer.isBuffer(data)

  sum.update(data, isBuffer ? 'binary' : 'utf8')
  return sum.digest(outEncode || 'hex')
}

export const cipher = function (
  mode,
  data,
  key = EMPTY_KEY,
  inEncode,
  outEncode
) {
  // 10.0.0之后, createCipher方法不推荐使用了
  if (VERSION >= 10.5) {
    return cipheriv(mode, data, key, EMPTY_KEY, inEncode, outEncode)
  }
  let isBuffer = Buffer.isBuffer(data)
  inEncode = isBuffer ? 'binary' : inEncode || 'utf8'
  outEncode = outEncode || 'base64'

  let cipher = crypto.createCipher(mode, key)
  let enStr = cipher.update(data, inEncode, outEncode)
  enStr += cipher.final(outEncode)
  if (AUTH_MODE.indexOf(mode) > -1) {
    let authTag = cipher.getAuthTag()
    return { enStr, authTag }
  }
  return enStr
}

export const decipher = function (
  mode,
  data,
  key = EMPTY_KEY,
  tag,
  inEncode,
  outEncode
) {
  // 10.0.0之后, createCipher方法不推荐使用了
  if (VERSION >= 10.5) {
    return decipheriv(mode, data, key, EMPTY_KEY, tag, inEncode, outEncode)
  }

  let isBuffer = Buffer.isBuffer(data)
  inEncode = isBuffer ? 'binary' : inEncode || 'base64'
  outEncode = outEncode || 'utf8'

  let cd = crypto.createDecipher(mode, key)
  if (AUTH_MODE.indexOf(mode) > -1) {
    cd.setAuthTag(tag)
  }
  let deStr = cd.update(data, inEncode, outEncode)
  deStr += cd.final(outEncode)
  return deStr
}

export const cipheriv = function (
  mode,
  data,
  key = EMPTY_KEY,
  iv = EMPTY_KEY,
  inEncode,
  outEncode
) {
  let isBuffer = Buffer.isBuffer(data)
  inEncode = isBuffer ? 'binary' : inEncode || 'utf8'
  outEncode = outEncode || 'base64'

  let cciv = crypto.createCipheriv(mode, key, iv)
  let enStr = cciv.update(data, inEncode, outEncode)
  enStr += cciv.final(outEncode)
  if (AUTH_MODE.indexOf(mode) > -1) {
    let authTag = cciv.getAuthTag()
    return { enStr, authTag }
  }
  return enStr
}

export const decipheriv = function (
  mode,
  data,
  key = EMPTY_KEY,
  iv = EMPTY_KEY,
  tag,
  inEncode,
  outEncode
) {
  let isBuffer = Buffer.isBuffer(data)
  inEncode = isBuffer ? 'binary' : inEncode || 'base64'
  outEncode = outEncode || 'utf8'

  let dcpiv = crypto.createDecipheriv(mode, key, iv)
  if (AUTH_MODE.indexOf(mode) > -1) {
    dcpiv.setAuthTag(tag)
  }
  let deStr = dcpiv.update(data, inEncode, outEncode)
  deStr += dcpiv.final(outEncode)
  return deStr
}
